{{
''***********************************************
''*  Program 10.1
''*  Author: Jon Titus 04-08-2013 Rev. 2
''*  Copyright 2014
''*  Released under Apache 2 license
''*  Display demonstration for a single 5-by-7
''*  LED matrix and 74HC595 shift registers that
''*  control columns and rows of LEDs.
''*  Test program to check LED connections.
''*  All LEDs should turn on.
''*  SPI_Asm object library used for SPI-type
''*  communications with the 74HC595 shift registers.
''*  LiteOn LTP-757G LED matrix module.
''***********************************************
}}

CON _clkmode = xtal1 + pll16x           'Set MCU clock operation
    _xinfreq = 5_000_000                'Set for 5 MHz crystal

OBJ
  SPI     :       "SPI_Asm"             'Invoke the SPI_Asm.spin file
                                        'saved in your Propeller working
                                        'file. Available from the Propeller
                                        'Object exchange (OBEX).
                                            
VAR
    byte pattern[10]                    'Storage for character patterns
    byte count                          'Counter used in display loop
    byte display_loop                   'Counter used for display-loop delay
    byte ser_data                       '8-bit serial data for LED columns
    byte data_port                      'Data-output pin to shift reg
    byte serial_clock                   'Pulse for shift reg clock
    byte register_clock                 'Pulse for register-transfer clock
    word digit_enable                   'Value to select column
    word data16_out                     '16-bit value for SPI transmission
                     
PUB Start                               'Main program starts here

pattern[0] := %11111111                 'Test pattern to light all LEDs

''SPI Setup
  SPI.start(15,0)                       'Set SPI for 1 uSec delay (15) and positive-
                                        'going clock pulses (0)
dira[8..10] := %111                     'Output pins for shift register
outa[8..10] := %000                     'Set shift-register outputs to 0

data_port := 8                          'Use P8 for shift-register data
serial_clock := 9                       'P9 for shift-register clock
register_clock := 10                    'P10 for register flip-flop outputs
                                        

'LOOP: Get pattern data for 5-by-7 LED matrix and send to shift register
'via SPI object. Transfer data to output flip-flops to drive LED rows,
'and control ULN2003A to turn on corresponding LED column.
 
  repeat                                              'Repeat this loop forever
    repeat display_loop from 0 to 59
        digit_enable := 1
        repeat count from 0 to 4                      'Loop for five columns
            ser_data := pattern[0]                    'Get test pattern
            data16_out := 0                           'Clear output word
            data16_out := (digit_enable << 8) | ser_data
           
            SPI.SHIFTOUT(data_port, serial_clock, SPI#MSBFIRST, 16, data16_out)
            digit_enable := digit_enable << 1         'Shift enable bit-

            waitcnt(clkfreq/100000 + cnt)             'Delay, wait for SPI to end
            outa[register_clock] := 1                 'Pulse RCLK input
            outa[register_clock] := 0                 'on 74HC595 IC
                                  
            waitcnt(clkfreq/2000 + cnt)               'Refresh time delay
            
 '-----End-----               

 