{{
''***********************************************
''*  Program 10.4
''*  Author: Jon Titus 04-09-2013 Rev. 4
''*  Copyright 2013
''*  Released under Apache 2 license
''*  Display demonstration that separates a
''*  A single 5-by-7 LED matrix uses 74HC595
''*  shift register to control columns and rows
''*  of LEDs to display characters.
''*  SPI_Asm object library used for SPI-type
''*  of 8-bit 7-segment patterns
''*  LUMEX LDQ-N516RI 4-digit display module.
''***********************************************
}}

CON _clkmode = xtal1 + pll16x           'Set MCU clock operation
    _xinfreq = 5_000_000                'Set for 5 MHz crystal

OBJ
  SPI     :       "SPI_Asm"             'Invoke the SPI_Asm.spin file
                                        'saved in your Propeller working
                                        'file. Available from the Propeller
                                        'Object exchange (OBEX)                                           
VAR
    byte ASCII_char                     'Variable for ASCII character
    byte char_patterns[320]             'Storage for character patterns
    byte message_loop                   'Counter for display columns
    byte display_buffer[5]              'Buffer-memory for temporary display data
    byte buffer_counter                 'Counter for buffer locations
    byte char_pointer                   'Calculated pointer to ASCII character
    byte count                          'Counter used in display loop
    byte display_loop                   'Counter used for display-loop delay
    byte ser_data                       '8-bit serial data for LED columns
    byte data_port                      'Data-output pin to shift reg
    byte serial_clock                   'Pulse for shift reg clock
    byte register_clock                 'Pulse for register-transfer clock
    word column_enable                  'Value to select display column to enable
    word data16_out                     '16-bit value for SPI transmission
                     
PUB Start                               'Main program starts here

        char_patterns[165]  :=  $7E       'Column 1, "A"
        char_patterns[166]  :=  $09
        char_patterns[167]  :=  $09
        char_patterns[168]  :=  $09
        char_patterns[169]  :=  $7E
        
        char_patterns[170]  :=  $7F       'Column 1, "B"
        char_patterns[171]  :=  $49
        char_patterns[172]  :=  $49
        char_patterns[173]  :=  $49
        char_patterns[175]  :=  $36
        
        char_patterns[175]  :=  $3E      'Column 1, "C"
        char_patterns[176]  :=  $41
        char_patterns[177]  :=  $41
        char_patterns[178]  :=  $41
        char_patterns[179]  :=  $22

        char_patterns[215]  :=  $7F      'Column 1, "K"
        char_patterns[216]  :=  $08
        char_patterns[217]  :=  $14
        char_patterns[218]  :=  $22
        char_patterns[219]  :=  $41

''SPI Setup
  SPI.start(15,0)                       'Set SPI for 1 uSec delay (15) and positive-
                                        'going clock pulses (0)
dira[8..10] := %111                     'Output pins for shift register
outa[8..10] := %000                     'Set shift-register outputs to 0

data_port := 8                          'Use P8 for shift-register data
serial_clock := 9                       'P9 for shift-register clock
register_clock := 10                    'P10 for register flip-flop outputs

 'Get the column data for a character and send to the shift register.
 'via SPI object. Transfer data to output flip-flops to drive rows,
 'and control ULN2003A to turn on corresponding column.

  
  repeat buffer_counter from 0 to 4         'Clear the display buffer memory
    display_buffer[buffer_counter] := 0
 
  
  repeat
    ASCII_char := 65                                            'ASCII code for "A"
    char_pointer := 5 * (ASCII_char - 32)                       'Calculate pointer
    repeat message_loop from 0 to 4                             'Count five columns in display
        
        repeat display_loop from 0 to 159                       'Longer character delay
            column_enable := 1
            repeat buffer_counter from 0 to 4                   'Loop for five columns
                ser_data := display_buffer[buffer_counter]
                data16_out := 0                                 'Clear output word
                data16_out := (column_enable << 8) | ser_data
           
                SPI.SHIFTOUT(data_port, serial_clock, SPI#MSBFIRST, 16, data16_out)
                column_enable := column_enable << 1             'Shift enable bit-

                waitcnt(clkfreq/100000 + cnt)                   'Delay, wait for SPI to end
                outa[register_clock] := 1                       'Pulse RCLK input
                outa[register_clock] := 0                       'on 74HC595 IC
                                  
                waitcnt(clkfreq/2000 + cnt)                     'Refresh time delay
        repeat buffer_counter from 0 to 4                       'Shift buffer contents
            display_buffer[buffer_counter] := display_buffer[buffer_counter + 1]
        display_buffer[4] := char_patterns[char_pointer]
        char_pointer++
        waitcnt(clkfreq/2000 + cnt)
        
 '-----End-----               

 