{{
''***********************************************
''*  Program 11.1
''*  Author: Jon Titus 11-05-2014 Rev. 2
''*  Copyright 2013
''*  Released under Apache 2 license
''*  Display-test demonstration for an 8-digit
''*  7-segment display (Lumex LDQ-N516RI)
''*  by a Maxim MAX7219 display controller IC.
''*  This test relies on 3.3-V-to-5-V logic-level
''*  conversion via a 74HC04 inverter IC.
''*  SPI_Asm object library used for SPI-type
''*  communications with the MAX7219 IC.
''*  Set for all-segments-on test.
''***********************************************
}}

CON _clkmode = xtal1 + pll16x           'Set MCU clock operation
    _xinfreq = 5_000_000                'Set for 5 MHz crystal

OBJ
  SPI     :       "SPI_Asm"             'Invoke the SPI_Asm.spin file
                                        'saved in your Propeller working
                                        'file. Available from the Propeller
                                        'Object exchange (OBEX).
                                            
VAR
    word MAX7219_init[10]               'Storage for initialization data
    word display_loop                   'Counter used for display loops
    byte data_port                      'Data-output pin to shift reg
    byte serial_clock                   'Pulse for shift reg clock
    byte chip_select                    'Pulse for LOAD pin
    word data16_out                     '16-bit value for SPI transmission
                     
PUB Start                               'Main program starts here

''SPI Setup
  SPI.start(15,0)                       'Set SPI for 1 uSec delay (15) and negative-
                                        'going clock pulses (0)
dira[8..10] := %111                     'Output pins for SPI and LOAD signals
outa[8..10] := %000                     'Set outputs to 0

data_port := 8                          'Use P8 for SPI data
serial_clock := 9                       'Use P9 for SPI clock
chip_select := 10                       'Use P10 for MAX7219 LOAD signal
                                        
'Insert your data byte in the array elements below
MAX7219_init[0] :=  $09__               'Decode Mode for "B Code"
MAX7219_init[1] :=  $0A__               'Intensity for half-bright LEDs
MAX7219_init[2] :=  $0B__               'Scan limit: Use all 8 digits
MAX7219_init[3] :=  $0C__               'Normal Operation
MAX7219_init[4] :=  $0F__               'Test all LEDs
               
'Initialize with MAX7219_init values
        
        outa[chip_select] := 0                  'Reset LOAD signal for MAX7219 

        repeat display_loop from 0 to 4         'Reset loop for five parameters                          
            data16_out := (MAX7219_init[display_loop])
            MAX7219_SPI_out

        'repeat display_loop from 0 to 7
         '   data16_out := ((display_loop + 1) <<8) + display_loop
         '   MAX7219_SPI_out
            
'SPI-output object, uses SPI_asm.spin file
'strobes MAX7219 LOAD pin after transmission
PUB  MAX7219_SPI_out

            
            SPI.SHIFTOUT(data_port, serial_clock, SPI#MSBFIRST, 16, data16_out)
            
            waitcnt(clkfreq/100000 + cnt)             'Delay, wait for SPI to end
                             
            outa[chip_select] := 1                 'LOAD pinto logic-1
            outa[chip_select] := 0                 'LOAD pin to logic-0
    
 '-----End-----               

 