{{
''***********************************************
''*  Program 12.2.spin
''*  Author: Jon Titus 04-29-2014 Rev. 2
''*  Copyright 2014
''*  Released under Apache 2 license
''*  Program used to obtain a 9-bit temperature value
''*  from a DS1620 Digital Thermometer and Thermostat.
''*  This program does not use the thermostat functions.
''*  Display temp on PST as temp degrees C with sign.
''*  Based on program DS1620_Demo, written by John
''*  Williams, Parallax, 28 March 2006.
''*  Uses P8X32A Propeller-I board.
''***********************************************
}}

CON

  _clkmode      = xtal1 + pll16x    'Set MCU clock operation
  _xinfreq      = 5_000_000         'Set for 5 MHz crystal

VAR
  ''Create variables for DS1620 control pins and serial output for the
  ''Parallax Serial Terminal (PST) for testing DS1620 operation.
  
  byte PropPin_DQ               'Variable for DS1620 DQ, pin 1
  byte PropPin_CLK              'Variable for DS1620 CLK, pin 2
  byte PropPin_RST              'Variable for DS1620 RST*, pin 3
  byte PropPin_SerOut           'Variable for P8X32A pin P30
  byte PropPin_SerIn            'Variable for P8X32A pin P31
  byte PropPin_SerMode          'Variable for P8X32A serial comm format
  word SerPort_BaudRate         'Variable for P8X32A serial comm bit rate         

OBJ

  temp  :  "ds1620_JT2"         'ds1620_JT2,spin file
  delay :  "timing"             'timing.spin file
  pst   :  "FullDuplexSerial"   'Serial library for testing

PUB main | tc

 PropPin_DQ        := 24        'P24 for DS1620 data pin; DQ pin1
 PropPin_CLK       := 25        'P25 for DS1620 clock pin; CLK pin 2
 PropPin_RST       := 26        'P26 for DS1620 reset pin; RST pin 3
 
 PropPin_SerOut    := 30        'P30 for USB transmit connection with host PC
 PropPin_SerIn     := 31        'P31 for USB receive connection with host PC
 PropPin_SerMode   := 0         'Invert RX mode. See FullDuplexSerial.spin.
 SerPort_BaudRate  := 9600      'Set baud rate at 9600 per second
  
    'Configure and start DS1620 temperature concersions
    temp.start(PropPin_DQ, PropPin_CLK, PropPin_RST)

    'Configure Propeller serial port to communicate with host PC via
    'Parallax Serial Terminal (PST)
    pst.Start(PropPin_SerIn, PropPin_SerOut, PropPin_SerMode, SerPort_BaudRate)

    'Loop to get temperature and display it as a hex value every
    'three seconds.
    repeat
        delay.pause1ms(3000)                '3 second delay    
        tc := temp.gettempc                 'Get temp via DS1620_JT2.spin               
        tc := tc & $1FF                     'Clear all bits except those from DS1620
        if tc > 255                         'Check MSB of temp data
            pst.str(String("-"))            'If MSB = 1, temp is negative, print minus
            tc := tc >> 1                   'Shift temp 1 bit to right
            tc := 1 + !(tc)                 'Get 2's complement of minus temp value
            
        else
            pst.str(String(" "))            'If MSB = 0, temp is positive, print a space
            tc := tc >> 1                   'Shift temp 1 bit to right
        tc := tc & $FF                      'Clear all bits except D7-D0
        pst.dec(tc)                         'Display temp (tc) as signed decimal on PST
                            
      pst.str (String("  Test..."))         'Some text to display
      pst.Tx(13)                            'Move cursor to start a new line
                                            'Run the repeat loop again and again...
'-----end of Program 12.2.spin-----
      