{{
''***********************************************
''*  Program 12.3.spin
''*  Author: Jon Titus 07-18-2014 Rev. 2
''*  Copyright 2014
''*  Released under Apache 2 license
''*  Program used to obtain a 9-bit temperature value
''*  from a DS1620 Digital Thermometer sensor and display
''*  the current temperature along with the minimum and
''*  maximum temperatures in the PST window.
''*  For the PST program, press F12 within Propeller Tool.
''*  Based on program DS1620_Demo, written by John
''*  Williams, Parallax, 28 March 2006.
''*  Uses P8X32A Propeller-I board.
''***********************************************
}}

CON

  _clkmode      = xtal1 + pll16x    'Set MCU clock operation
  _xinfreq      = 5_000_000         'Set for 5 MHz crystal

VAR
  ''Create variables for DS1620 control pins and serial output for the
  ''Parallax Serial Terminal (PST) for testing DS1620 operation.
  
  byte PropPin_DQ               'Variable for DS1620 DQ, pin 1
  byte PropPin_CLK              'Variable for DS1620 CLK, pin 2
  byte PropPin_RST              'Variable for DS1620 RST*, pin 3
  byte PropPin_SerOut           'Variable for P8X32A pin P30
  byte PropPin_SerIn            'Variable for P8X32A pin P31
  byte PropPin_SerMode          'Variable for P8X32A serial comm format
  word SerPort_BaudRate         'Variable for P8X32A serial comm bit rate         
  long Current_Temp             'Long variable for signed temperature value
  long Temp_Max                 'Long variable for maximum temp value
  long Temp_Min                 'Long variable for minimum temp value
OBJ

  'You need these files, too, in your working folder
  temp  :  "ds1620_JT2"         'ds1620_JT2,spin file
  delay :  "timing"             'timing.spin file
  pst   :  "FullDuplexSerial"   'Serial library for testing

PUB main | tc

 PropPin_DQ        := 24        'P24 for DS1620 data pin; DQ pin1
 PropPin_CLK       := 25        'P25 for DS1620 clock pin; CLK pin 2
 PropPin_RST       := 26        'P26 for DS1620 reset pin; RST pin 3
 
 PropPin_SerOut    := 30        'P30 for USB transmit connection with host PC
 PropPin_SerIn     := 31        'P31 for USB receive connection with host PC
 PropPin_SerMode   := 0         'Invert RX mode. See FullDuplexSerial.spin.
 SerPort_BaudRate  := 9600      'Set baud rate at 9600 per second
  
    'Configure and start DS1620 temperature concersions
    temp.start(PropPin_DQ, PropPin_CLK, PropPin_RST)

    'Configure Propeller serial port to communicate with host PC via
    'Parallax Serial Terminal (PST)
    pst.Start(PropPin_SerIn, PropPin_SerOut, PropPin_SerMode, SerPort_BaudRate)

    'Loop to get temperature and display it as a hex value every
    'three seconds.
    Temp_Max := -55                         'Set max temp to lowest to start
    Temp_Min := 125                         'Set min temp to highest to start
    repeat
        delay.pause1ms(3000)                '3 second delay    
        tc := temp.gettempc                 'Get temp via DS1620_JT2.spin               
        tc := $FF & (tc >> 1)               'Shift tc one bit to the left and
                                            'Clear all bits except D7-D0
        Current_Temp := ~tc                 'Sign-extend tc into Current_Temp                 
        pst.dec(Current_Temp)               'Print Current_temp as decimal value
        pst.str (String("  Test 3..."))     'Some text to display
        if (Current_Temp > Temp_Max)        'Compare current to highest
            Temp_Max := Current_Temp
        if (Current_Temp < Temp_Min)        'Compare current to lowest
            Temp_Min := Current_Temp
        pst.dec(Temp_Max)                   'Print maximum temperature
        pst.str (String("  Max    "))
        pst.dec(Temp_Min)                   'Print minimum temperature
        pst.str (String("  Min"))
        pst.Tx(13)                          'Move cursor to start a new line
                                            'Run the repeat loop again and again...
'-----end of Program 12.3.spin-----
      