{{
''***********************************************
''*  Program 13.1
''*  Author: Jon Titus 04-08-2014 Rev. 2
''*  Copyright 2014
''*  Released under Apache 2 license
''*  Display demonstration for a single 5-by-7
''*  Simple control program for MAX7219 control of
''*  8-digit 7-segment display module with common-
''*  cathode configuration.
''*  Digits get numbered with their position value.
''*  SPI_Asm object library used for SPI-type
''*  communications with the 74HC595 shift registers.
''*  LiteOn LTP-757G LED matrix module.
''***********************************************
}}

CON _clkmode = xtal1 + pll16x           'Set MCU clock operation
    _xinfreq = 5_000_000                'Set for 5 MHz crystal

OBJ
  SPI     :       "SPI_Asm"             'Invoke the SPI_Asm.spin file
                                        'saved in your Propeller working
                                        'file. Available from the Propeller
                                        'Object exchange (OBEX).
                                            
VAR
    word MAX7219_init[10]               'Storage for initialization data
    word display_loop                   'Counter used for display loops
    byte data_port                      'Data-output pin to shift reg
    byte serial_clock                   'Pulse for shift reg clock
    byte chip_select                    'Pulse for LOAD pin
    word data16_out                     '16-bit value for SPI transmission
    byte digit[6]
    
PUB Start                               'Main program starts here

''SPI Setup
  SPI.start(15,0)                       'Set SPI for 1 uSec delay (15) and negative-
                                        'going clock pulses (0)
dira[8..10] := %111                     'Output pins for SPI and LOAD signals
outa[8..10] := %000                     'Set outputs to 0

data_port := 8                          'Use P8 for SPI data
serial_clock := 9                       'Use P9 for SPI clock
chip_select := 10                       'Use P10 for MAX7219 LOAD signal
                                        
MAX7219_init[0] :=  $0900               'No character decoding, segment control
MAX7219_init[1] :=  $0A07               'Intensity = 0x07 for half-bright LEDs
MAX7219_init[2] :=  $0B07               'Scan limit: Use all 8 digits
MAX7219_init[3] :=  $0C01               'Normal Operation = $0C01, Shutdown = $0C00
MAX7219_init[4] :=  $0F00               'Test? $0F00 = no; $0F01 = yes.

digit[0] := $37
digit[1] := $30
digit[2] := $01
digit[3] := $3C
digit[4] := $1D
digit[5] := $15

                    
'Initialize MAX7219 IC with MAX7219_init values
        
        outa[chip_select] := 0                  'Reset LOAD signal for MAX7219 

        repeat display_loop from 0 to 4         'Reset loop for five parameters                          
            data16_out := (MAX7219_init[display_loop])
            MAX7219_SPI_out

        'Clear all 8 digits
        repeat display_loop from 0 to 7
            data16_out := ((display_loop + 1)  <<8) + $00
            MAX7219_SPI_out

        repeat display_loop from 0 to 5
            data16_out := ((display_loop + 1)  <<8) + digit[display_loop]
            MAX7219_SPI_out
        

        
            
'SPI-output object, uses SPI_asm.spin file
'strobes MAX7219 LOAD pin after transmission
PUB  MAX7219_SPI_out

            
            SPI.SHIFTOUT(data_port, serial_clock, SPI#MSBFIRST, 16, data16_out)
            
            waitcnt(clkfreq/100000 + cnt)               'Delay, wait for SPI to end
                             
            outa[chip_select] := 1                      'LOAD pin to logic-1
            outa[chip_select] := 0                      'LOAD pin to logic-0
    
 '-----End-----               

 