{{
''***********************************************
''*  Program 13.3 Display_MAX7219 objects
''*  Author: Jon Titus 11-12-2014 Rev. 2
''*  Copyright 2013
''*  Released under Apache 2 license
''*  Spin objects to start and clear a MAX7219
''*  display-driver IC and eight 7-segment displays.
''*  The Display_MAX7219 object takes values from
''*  an array and displays them.
''*  SPI_Asm object library used for SPI-type
''*  communication.
''***********************************************
}}

CON _clkmode = xtal1 + pll16x           'Set MCU clock operation
    _xinfreq = 5_000_000                'Set for 5 MHz crystal

OBJ
  SPI     :       "SPI_Asm"             'Invoke the SPI_Asm.spin file
                                        'saved in your Propeller working
                                        'file. Available from the Propeller
                                        'Object exchange (OBEX).
                                            
VAR
    word MAX7219_init[10]               'Storage for initialization data
    word display_loop                   'Counter used for display loops
    byte data_port                      'Data-output pin to shift reg
    byte serial_clock                   'Pulse for shift reg clock
    byte chip_select                    'Pulse for LOAD pin
    word data16_out                     '16-bit value for SPI transmission
    
'=====================
PUB Start_MAX7219                       'Start object to set up variables,
                                        'pin assignments, and initialization data
''SPI Setup
  SPI.start(15,0)                       'Set SPI for 1 uSec delay (15) and negative-
                                        'going clock pulses (0)
dira[8..10] := %111                     'Output pins for SPI and LOAD signals
outa[8..10] := %000                     'Set outputs to 0

data_port := 8                          'Use P8 for SPI data
serial_clock := 9                       'Use P9 for SPI clock
chip_select := 10                       'Use P10 for MAX7219 LOAD signal
                                        
'MAX7219 initialization data saved here:
MAX7219_init[0] :=  $09FF               'Decode Mode for "B Code"
MAX7219_init[1] :=  $0A07               'Intensity = 0x07 for half-bright LEDs
MAX7219_init[2] :=  $0B07               'Scan limit: Use all 8 digits
MAX7219_init[3] :=  $0C01               'Normal Operation = $0C01, Shutdown = $0C00
MAX7219_init[4] :=  $0F00               'Test? $0F00 = no; $0F01 = yes.

'Initialize MAX7219 IC with MAX7219_init values
        
        outa[chip_select] := 0                  'Reset LOAD signal for MAX7219 

        repeat display_loop from 0 to 4         'Send MAX7219 five parameters                          
            data16_out := (MAX7219_init[display_loop])
            MAX7219_SPI_out

'=====================
'Clear all 8 digits controlled by MAX7219
PUB Clear_MAX7219
        repeat display_loop from 0 to 7                    'Send MAX7219 register number
            data16_out := ((display_loop + 1)  <<8) + $0F  'and value $0F to blank all
            MAX7219_SPI_out                                'displays

'=====================
'Display values saved in an array
PUB Display_MAX7219(ptr)                                   'use array pointer to access
        repeat display_loop from 2 to 5                    'values in an array
            data16_out := ((display_loop + 1)  <<8) + byte[ptr][display_loop]
            MAX7219_SPI_out
        
'=====================
'SPI-output object, uses SPI_asm.spin file
'strobes MAX7219 LOAD pin after transmission
PUB  MAX7219_SPI_out

            
            SPI.SHIFTOUT(data_port, serial_clock, SPI#MSBFIRST, 16, data16_out)
            
            waitcnt(clkfreq/100000 + cnt)               'Delay, wait for SPI to end
                             
            outa[chip_select] := 1                      'LOAD pin to logic-1
            outa[chip_select] := 0                      'LOAD pin to logic-0
    
 '-----End-----               

 