{{
''***********************************************
''*  Program 13.4 Digital Thermometer Program
''*  Author: Jon Titus 11-12-2014 Rev. 2
''*  Copyright 2013
''*  Released under Apache 2 license
''*  Display demonstration for a digital thermometer
''*  Used DS1620 IC as a temperature sensor and a 
''*  MAX7219 display-driver IC to display Celsius
''*  temperatures
''*  SPI_Asm object library used for SPI-type
''*  communication
''***********************************************
}}

CON _clkmode = xtal1 + pll16x           'Set MCU clock operation
    _xinfreq = 5_000_000                'Set for 5 MHz crystal

OBJ

  MAX7219   :  "Program 13.3"       'Objects here control MAX7219
  temp      :  "ds1620_JT2"         'ds1620_JT2,spin file for sensor
  delay     :  "timing"             'timing.spin file
  
  
VAR
  byte digit[6]                 'Array to save temperature digits
  byte PropPin_DQ               'Variable for DS1620 DQ, pin 1
  byte PropPin_CLK              'Variable for DS1620 CLK, pin 2
  byte PropPin_RST              'Variable for DS1620 RST*, pin 3        
  long Current_Temp             'Long variable for signed temperature value
  byte hundreds                 'Hundreds value storage
  byte tens                     'Tens value storage
  
PUB main | tc

MAX7219.Start_MAX7219           'Initialize the MAX7219 display driver IC
MAX7219.Clear_MAX7219           'Turn all digits off

 PropPin_DQ        := 24        'P24 for DS1620 data pin; DQ pin1
 PropPin_CLK       := 25        'P25 for DS1620 clock pin; CLK pin 2
 PropPin_RST       := 26        'P26 for DS1620 reset pin; RST pin 3
 
    'Configure and start DS1620 temperature conversions
    temp.start(PropPin_DQ, PropPin_CLK, PropPin_RST)

'Rhis loop gets a new temperature and display it
' as a decimal value every four seconds.
    
    repeat
        delay.pause1ms(2000)                '2 second delay    
        tc := temp.gettempc                 'Get temp "tc" via DS1620_JT2.spin
        delay.pause1ms(2000)                '2 second delay   
        tc := $FF & (tc >> 1)               'Shift tc one bit to the left (divide by 2)

        if (tc > 127)                       'Look at MSB for temperature sign
            digit[2] := 10                  'Negative temperature, so set digit 2
                                            'to display a minus
            tc := 1 + !(tc)                 'Get 2's complement of minus temp value
            
        else                                'If temperature is positive, blank
            digit[2] := 15                  'digit 2 on display.
        
        tc := tc & %01111111                'Mask out the MSB (sign bit), set it to 0
        hundreds := tc/100                  'Divide temperature value by 100'
        
        if (hundreds == 0)                  'Test for temp of 100 or greater
                digit[3] := 15              'No 100's, so blank digit 3
        else
                digit[3] := 1               'Can only have a 1 in hundreds, so set
                                            'digit 3 to "1"
                tc := tc - 100              'Then subtract 100 from temperature value
                
        tens := tc/10                       'Check for no 10s and no 100s in temp
        if (tens == 0) AND (hundreds == 0)
            digit[4] := $0F                 'No tens AND no hundreds, blank
                                            'ten's digit
        else
             digit[4] := tens               'OK, display non-zero ten's digit
             
        tc := tc - (tens * 10)              'Get 1s value
        digit[5] := tc                      'Set digit 5 to 1s value
        
        MAX7219.Clear_MAX7219               'Clear the display               
        MAX7219.Display_MAX7219(@digit)     'Display latest temperature
'-----end-----