{{
''***********************************************
''*  Program 14.1 Load DS1620 TH and TL registers
''*  Author: Jon Titus 08-13-2014 Rev. 1
''*  Copyright 2014
''*  Released under Apache 2 license
''*  Demonstration of the DS1620 as a thermostat
''*  Three LEDs indicate over-temperature,
''*  under-temperature, and range "excursion"
''***********************************************
}}

CON _clkmode = xtal1 + pll16x       'Set MCU clock operation
    _xinfreq = 5_000_000            'Set for 5 MHz crystal

    Write_TH        = $01           'Write command for 9-bit TH register       
    Write_TL        = $02           'Write commend for 9-bit TL register
    Read_TH         = $A1           'Read from 9-bit TH register
    Read_TL         = $A2           'Read from 9-bit TL register
    Nine_bit_data   = 17            '17 bits for command plus data
    Eight_bit_data  = 16            '16 bits for command plus data
OBJ

  temp      :  "DS1620_JT2"         'ds1620_JT2,spin file
  delay     :  "timing"             'timing.spin file
  pst       :  "FullDuplexSerial"   'Serial library for testing
  
VAR
  
  byte PropPin_DQ               'Variable for DS1620 DQ, pin 1
  byte PropPin_CLK              'Variable for DS1620 CLK, pin 2
  byte PropPin_RST              'Variable for DS1620 RST*, pin 3
  byte PropPin_SerOut           'Variable for P8X32A pin P30
  byte PropPin_SerIn            'Variable for P8X32A pin P31
  byte PropPin_SerMode          'Variable for P8X32A serial comm format
  word SerPort_BaudRate         'Variable for P8X32A serial comm bit rate         
  long TH_value                 'TH register value
  long TL_value                 'TL register value
  
PUB main | tc

 PropPin_DQ        := 24        'P24 for DS1620 data pin; DQ pin1
 PropPin_CLK       := 25        'P25 for DS1620 clock pin; CLK pin 2
 PropPin_RST       := 26        'P26 for DS1620 reset pin; RST pin 3
 
 PropPin_SerOut    := 30        'P30 for USB transmit connection with host PC
 PropPin_SerIn     := 31        'P31 for USB receive connection with host PC
 PropPin_SerMode   := 0         'Invert RX mode. See FullDuplexSerial.spin.
 SerPort_BaudRate  := 9600      'Set baud rate at 9600 per second

 'Put TH and TL values here:
 TH_value          := $003C     'Your upper-limit temperature, 9 bits
 TL_value          := $001E     'Your lower-limit temperature, 9 bits
  
'Configure and start DS1620 temperature conversions
    temp.start(PropPin_DQ, PropPin_CLK, PropPin_RST)

'Configure Propeller serial port to communicate with host PC via
'Parallax Serial Terminal (PST)
    pst.Start(PropPin_SerIn, PropPin_SerOut, PropPin_SerMode, SerPort_BaudRate)

'Write data to TH and TL registers
    temp.SendDS1620(Write_TH, TH_value, Nine_bit_data)
    temp.SendDS1620(Write_TL, TL_value, Nine_bit_data)

'Loop to get temperature and display it as a hex value every four seconds.     
    repeat
        delay.pause1ms(4000)    '4 second delay    
        tc := temp.gettempc     'Get temp via DS1620_JT2.spin
        pst.hex(tc, 3)          'Display 3-digit hex temperature
        pst.Tx(13)              'Force a new line on PST
           
'' -----End Program 14.1-----        
        
        
  