{{
''***********************************************
''*  Program 14.2 Load DS1620 TH and TL registers
''*  Author: Jon Titus 12-05-2014 Rev. 1
''*  Copyright 2014
''*  Released under Apache 2 license
''*  Demonstration of the DS1620 THF and TLF
''*  flags. Messages printed when flags set. After
''*  printing messages, flags get reset.
''***********************************************
}}

CON _clkmode = xtal1 + pll16x       'Set MCU clock operation
    _xinfreq = 5_000_000            'Set for 5 MHz crystal

    'DS1620 commands
    Read_Temp         = $AA         ' read temperature
    Read_Counter      = $A0         ' read counter
    Read_Slope        = $A9         ' read slope
    Start_Convert     = $EE         ' start conversion
    Stop_Convert      = $22         ' stop conversion
    Write_Config      = $0C         ' write config register
    Read_Config       = $AC         ' read config register
    Write_TH          = $01         ' write command for 9-bit TH register       
    Write_TL          = $02         ' write commend for 9-bit TL register
    Read_TH           = $A1         ' read from 9-bit TH register
    Read_TL           = $A2         ' read from 9-bit TL register

    Nine_bit_data     = 17          '17 bits for command plus data
    Eight_bit_data    = 16          '16 bits for command plus data
    
OBJ

  temp      :  "DS1620_JT3"         'ds1620_JT2,spin file
  delay     :  "timing"             'timing.spin file
  pst       :  "FullDuplexSerial"   'Serial library for testing
  
VAR
  
  byte PropPin_DQ               'Variable for DS1620 DQ, pin 1
  byte PropPin_CLK              'Variable for DS1620 CLK, pin 2
  byte PropPin_RST              'Variable for DS1620 RST*, pin 3
  byte PropPin_SerOut           'Variable for P8X32A pin P30
  byte PropPin_SerIn            'Variable for P8X32A pin P31
  byte PropPin_SerMode          'Variable for P8X32A serial comm format
  word SerPort_BaudRate         'Variable for P8X32A serial comm bit rate         
  long TH_value                 'TH register value
  long TL_value                 'TL register value
  byte DS1620_flags             'Variable for DS1620 Config Reg data
  byte Config_Reset
  
PUB main | tc

 PropPin_DQ        := 24        'P24 for DS1620 data pin; DQ pin1
 PropPin_CLK       := 25        'P25 for DS1620 clock pin; CLK pin 2
 PropPin_RST       := 26        'P26 for DS1620 reset pin; RST pin 3
 
 PropPin_SerOut    := 30        'P30 for USB transmit connection with host PC
 PropPin_SerIn     := 31        'P31 for USB receive connection with host PC
 PropPin_SerMode   := 0         'Invert RX mode. See FullDuplexSerial.spin.
 SerPort_BaudRate  := 9600      'Set baud rate at 9600 per second

 'Put TH and TL values here:
 TH_value          := $003C     'Your upper-limit temperature, 9 bits
 TL_value          := $001E     'Your lower-limit temperature, 9 bits
 Config_Reset      := %00000010 

'Configure and start DS1620 temperature conversions
    temp.start(PropPin_DQ, PropPin_CLK, PropPin_RST)

'Configure Propeller serial port to communicate with host PC via
'Parallax Serial Terminal (PST)
    pst.Start(PropPin_SerIn, PropPin_SerOut, PropPin_SerMode, SerPort_BaudRate)

'Write data to TH and TL registers
    temp.SendDS1620(Write_TH, TH_value, Nine_bit_data)
    temp.SendDS1620(Write_TL, TL_value, Nine_bit_data)

'Loop to get temperature and display it as a hex value every four seconds.     
    repeat
        delay.pause1ms(4000)                                    '4 second delay    
        tc := temp.gettempc                                     'Get temp via DS1620_JT2.spin
        pst.hex(tc, 3)                                          'Display 3-digit hex temperature
        pst.str(String("   "))                                  'insert some spaces
        DS1620_flags := temp.GetDS1620_data (Read_Config, 8)    'Read Config Register
        DS1620_flags &= %01100000                               'Mask all but D6 and D5

        if DS1620_flags > 32                                    'If D6 = 1 (64)
            DS1620_flags -= 64                                  'Subtract 64
            pst.str(String("THF Set"))                          'Print message
        
        if DS1620_flags == 32                                   'If D5 = 1 (32) 
            pst.str(String("TLF Set"))                          'Print message
        pst.Tx(13)                                              'New line for PST

        'Reset Configuration Register
        temp.SendDS1620(Write_Config, Config_Reset, Eight_bit_data)    
           
'' -----end Program 14.2-----        
        
        
  