{{
''***********************************************
''*  Program 16.2 Ramp Analog-to-Digital Converter
''*  Author: Jon Titus 09-26-2014 Rev. 1
''*  Copyright 2014
''*  The Propeller created an increasing voltage
''*  that a comparator IC compares with an unknown
''*  voltage from a variable resistor. When a match
''*  occurs, the ramp stops and repeats the
''*  process again and again.
''*  Released under Apache 2 license
''*  Uses Parallax P8X32A Propeller board.
''***********************************************
}}

CON

  _clkmode      = xtal1 + pll16x    'Set MCU clock operation
  _xinfreq      = 5_000_000         'Set for 5 MHz crystal

VAR

  byte Counter                        'Variable for loop counter and DAC output

OBJ

  delay :   "timing"              

PUB main

 dira[15..8] := %11111111           'Set P15--P8 as output pins for DAC
 outa[15..8] := %00000000           'Set pins P15--P8 as 0's to clear DAC

 dira[24] := %1                     'Set P24 as an output pin to latch DAC data
 outa[24] := %1                     'Set pin P24 as logic 1

 dira[25] := %0                     'Set pin P25 as an input pin to test
                                    'comparator output.
 dira[23..16] := %11111111          'Set LEDs as outputs
 
'Code to initialize the DAC
 Counter     := 0                       'Set Counter to zero
 outa[15..8] := Counter                 'Reset Counter to 0 for 0 volts out of DAC
 outa[24]    := %0                      'Load DAC with data, set pin 24 as logic 0                
 outa[24]    := %1                      'Take pin 24 back to a logic 1

 'Loop that performs continuous analog-to-digital conversions
    repeat
      
        if  ina[25] <> 0                'Check comparator output, no match, so keep going
            outa[15..8] := Counter      'Output Counter value on P15--P8
            outa[24]    := 0            'Load DAC with data, set pin 24 as logic 0                
            outa[24]    := 1            'Take pin 24 back to a logic 1
            Counter++                   'Increment Counter by one

        else                            'Equal voltages detected, end of conversion!
            outa[23..16] := Counter     'Display on LEDs the DAC value that caused a match
            Counter     := 0            'Reset Counter to 0
            outa[15..8] := Counter      'Reset DAC to 0 volts out
            outa[24]    := %0           'Load DAC with data, set pin 24 as logic 0                
            outa[24]    := %1           'Take pin 24 back to a logic 1
            outa[26]    := %0           'Create a pulse to trigger scope
            outa[26]    := %1           
                                        'Go back to repeat loop again
 'end of Program 16.2
    
 