{{
''***********************************************
''*  Program 16.3 Ramp Analog-to-Digital Converter
''*  Author: Jon Titus 09-26-2014 Rev. 1
''*  Copyright 2014
''*  The Propeller creates an increasing voltage
''*  that a comparator IC compares with an unknown
''*  voltage from a variable resistor. When a match
''*  occurs, the ramp stops and the program displays
''*  the DAC's 8-bit value on eight LEDs and puts
''*  the decimal value in the PST window. Then the
''*  process repeats again and again.
''*  Released under Apache 2 license
''*  Uses Parallax P8X32A Propeller board.
''***********************************************
}}

CON

  _clkmode      = xtal1 + pll16x    'Set MCU clock operation
  _xinfreq      = 5_000_000         'Set for 5 MHz crystal

VAR

  byte Counter                      'Variable for loop counter and DAC output
  byte PropPin_SerOut               'Variable for P8X32A pin P30
  byte PropPin_SerIn                'Variable for P8X32A pin P31
  byte PropPin_SerMode              'Variable for P8X32A serial comm format
  word SerPort_BaudRate             'Variable for P8X32A serial comm bit rate
  
OBJ

  delay :   "timing"              
  pst   :   "FullDuplexSerial"
  
PUB main

 dira[15..8] := %11111111           'Set P15--P8 as output pins for DAC
 outa[15..8] := %00000000           'Set pins P15--P8 as 0's to clear DAC

 dira[24] := %1                     'Set P24 as an output pin to latch DAC data
 outa[24] := %1                     'Set pin P24 as logic 1

 dira[25] := %0                     'Set pin P25 as an input pin to test
                                    'comparator output.

  dira[23..16] := %11111111         'Set LED control pins as outputs
  outa[23..16] := 0                 'Clear LEDs, turn them off

'Configuration information for serial communications on P8X32A board
  PropPin_SerOut    := 30           'P30 for USB transmit connection with host PC
  PropPin_SerIn     := 31           'P31 for USB receive connection with host PC
  PropPin_SerMode   := 0            'Invert RX mode. See FullDuplexSerial.spin.
  SerPort_BaudRate  := 9600         'Set baud rate at 9600 per second 

'Configure Propeller serial port to communicate with host PC via
'Parallax Serial Terminal (PST)
 pst.Start(PropPin_SerIn, PropPin_SerOut, PropPin_SerMode, SerPort_BaudRate)
 
'Code to initialize the DAC
 Counter     := 0                       'Set Counter to zero
 outa[15..8] := Counter                 'Reset Counter to 0 for 0 volts out of DAC
 outa[24]    := %0                      'Load DAC with data, set pin 24 as logic 0                
 outa[24]    := %1                      'Take pin 24 back to a logic 1

 'Loop that performs continuous analog-to-digital conversions
    repeat
      delay.pause1ms(10)                'Short delay
        if  ina[25] <> 0                'Check comparator output, no match, so keep going
            outa[15..8] := Counter      'Output Counter value on P15--P8
            outa[24]    := 0            'Load DAC with data, set pin 24 as logic 0                
            outa[24]    := 1            'Take pin 24 back to a logic 1
            Counter++                   'Increment Counter by one

        else                            'Equal voltages detected, end of conversion!
            outa[23..16] := Counter     'Send binary data to LEDs
            pst.str(String("ADC = "))   'Print text to PST window
            pst.dec(Counter)            'Print ADC value as decimal
            pst.tx(13)                  'Create a new line in PST window
            
            Counter     := 0            'Reset Counter to 0
            outa[15..8] := Counter      'Reset DAC to 0 volts out
            outa[24]    := %0           'Load DAC with data, set pin 24 as logic 0                
            outa[24]    := %1           'Take pin 24 back to a logic 1
            outa[26]    := %0           'Create a pulse to trigger scope
            outa[26]    := %1           
                                        'Go back to repeat loop again
 'end of Program 16.3
    
 