{{
''***********************************************
''*  Program 16.4
''*  Author: Jon Titus 09-27-2014 Rev. 1
''*  Copyright 2014
''*  Successive-approximation analog-to-
''*  conversion. Uses LTC1450L as an 8-bit DAC
''*  with only 8 MSBs used. LSBs grounded.
''*  Results shown on LEDs.
''*  Released under Apache 2 license
''*  Uses Parallax P8X32A Propeller board.
''***********************************************
}}

CON

  _clkmode      = xtal1 + pll16x    'Set MCU clock operation
  _xinfreq      = 5_000_000         'Set for 5 MHz crystal

VAR

byte    Test_Pattern                'Test pattern for DAC              
byte    ADC_Result                  'Result of conversion  
byte    Numb_of_Bits                'Number of bits on DAC

OBJ

  delay :   "timing"
  
PUB main

 dira[15..8] := %11111111           'Set P15--P8 as output pins to DAC
 outa[15..8] := %00000000           'Set pins P15--P8 as 0's to clear DAC

 dira[24] := %1                     'Set P24 as an output pin to latch DAC data
 outa[24] := %1                     'Set pin P24 as logic 1

 dira[25] := %0                     'Set pin P25 as an input pin to test
                                    'comparator output.

 dira[23..16] := %11111111          'Set LED control pins as outputs
 outa[23..16] := 0                  'Clear LEDs, turn them off

 Numb_of_Bits := 8                  'Set number of bits for DAC
 
 'Code to initialize the DAC for each conversion
 repeat
    Test_Pattern     := %10000000                       'First test pattern for ADC
    ADC_Result  := 0                                    'First DAC output for 0 volts
    outa[15..8] := 0                                    'Load DAC with ADC_Result
    outa[24]    := %0                                   'Set /LDAC to logic 0                
    outa[24]    := %1                                   'Take /LDAC to logic 1
  
        repeat Numb_of_Bits                             'One pass through loop for each bit                             
            delay.pause1ms(1000)
            ADC_Result := ADC_Result + Test_Pattern
            outa[15..8] := ADC_Result                   'Load DAC with ADC_Result
            outa[24]    := %0                           'Set /LDAC to logic 0                
            outa[24]    := %1                           'Take /LDAC to logic 1

            if ina[25] <> 1                             'Comparator signals too high
                ADC_Result := ADC_Result - Test_Pattern 'Remove present test bit from result
                outa[24]    := %0                       'Set /LDAC to logic 0                
                outa[24]    := %1                       'Take /LDAC to logic 1
      
            Test_Pattern := Test_Pattern >> 1           'Shift test pattern 1 bit to right            

    outa[23..16] := ADC_Result                          'Display result on LEDs
        
 'end of Program 16.4  
    
 