''*****************************************************
''*  Program 17.3, Analog-to-Digital Converter Test
''*  Based on code created by Ken Gracey, Parallax
''*  Modified by John Abshier (jabshier on Forum)
''*  with different pin numbers and potentiometers
''*  instead of joystick inputs.
''*  Modified by Jon Titus, 09-28-2014 Rev. 1
''*  Copyright 2014
''*  Software for a Microchip Technology MCP3202 12-bit       
''*  ADC that displays the rounded Kelvin temperature
''*  from am LM335 temperature sensor.                               
''*****************************************************
OBJ

  pst       : "FullDuplexSerial"    'serial communication objects here    
  MCP3202   : "MCP3202"             'MCP3202 driver objects here
  delay     : "timing"              'timing.spin file
  
CON

  _CLKMODE      = XTAL1 + PLL4X     'set MCU clock operations
  _XINFREQ      = 5_000_000         '5MHz Crystal

  clock_pin         = 24            'Pin assignments for connections to            '         
  data_pin          = 23            'the MCP3202 ADC from a Propeller
  chip_select_pin   = 25            'P8X32A MCU board
                
VAR    

  long  CH0                         'Variable assignment for Channel-0 data
  long  CH1                         'Variable assignment for Channel-1 data

  byte PropPin_SerOut               'Variable for P8X32A pin P30
  byte PropPin_SerIn                'Variable for P8X32A pin P31
  byte PropPin_SerMode              'Variable for P8X32A serial comm format
  word SerPort_BaudRate             'Variable for P8X32A serial comm bit rate
  long Kelvin_Raw_Data              'Variable for temperature data to round
  long Kelvin_Rounded_Data          'Variable for rounded temperature
  
PUB Main

 PropPin_SerOut    := 30            'P30 for USB transmit connection with host PC
 PropPin_SerIn     := 31            'P31 for USB receive connection with host PC
 PropPin_SerMode   := 0             'Invert RX mode. See FullDuplexSerial.spin.
 SerPort_BaudRate  := 9600          'Set baud rate at 9600 per second

 'Serial-communication start-up operation
 pst.Start(PropPin_SerIn, PropPin_SerOut, PropPin_SerMode, SerPort_BaudRate)

 'MCP3202 ADC start-up configuration
 'Enable Channels 0 and 1 for single-ended analog inputs
 'See MCP3202 datasheet for configuration options
 MCP3202.start(data_pin, clock_pin, chip_select_pin, %0011)
                                            
repeat
  pst.tx(1)                                 'Move cursor to home position in PST window   
  pst.str(string("MCP3202 12-bit A/D"))     'Display this string in the PST window
  pst.Tx(13)                                'Move cursor to a new line
  pst.str(string("=================="))     'Display this string in the PST window 
  pst.Tx(13)                                'Move cursor to a new line                

  CH0 := MCP3202.in(0)                      'Get digital value for analog signal on Channel 0
  pst.str(string("Channel 0: "))          'Display this string in the PST window    
  pst.tx(11)                                'Clear previous Channel 0 value from PST window
  pst.dec(CH0)                              'Display Channel-0 decimal value
  pst.Tx(13)                                'Move cursor to a new line

  CH1 := MCP3202.in(1)                      'Get digital value for analog signal on Channel 1
  Kelvin_Raw_Data := 330 * CH1/2048
  Kelvin_Rounded_Data := (Kelvin_Raw_Data + 1) / 2
  pst.str(string("Temp in K: "))            'Display this string in the PST window
  pst.tx(11)                                'Clear previous Channel 1 value from PST window
  pst.dec(Kelvin_Rounded_Data)                              'Display Channel-1 decimal value
  pst.Tx(13)                                'Move cursor to a new line

  delay.pause1ms(100)                       'Cause a short delay in milliseconds

  '-----end Program 17.3-----      