''*****************************************************
''*  Program ir_oscope.spin, Rev. 2.
''*  Created by Bob Belleville
''*  This object receives input from an IR remote
''*  and produces a stream of "." and "0" on the Parallax
''*  Serial Terminal to give a quick and simple check of
''*  Propeller connections and the IR remote.
''*  Using a period instead of a 1 makes the 0s stand out.
''*  Modified by Jon Titus 10-17-2013 and re-commented.
''*  Period replaced with a dash.
''*  See the readme.pdf file in the IR Kit OBEX zip file
''*  for more documentation.
''*****************************************************
{{
''***********************************************
''*  Program version EX18.1.spin
''*  Author: Jon Titus 12-02-2013 Rev. 2
''*  Titus Revisions Copyright 2013
''*  Substituted a dash "-" for the period in original
''*  Fixed problem with synchronization of serial-receive
''*  code to ensure stable results.
''*  Uses P8X32A Propeller-I board.
''*  SET BAUD RATE AT 9600 IN PST!!!
''***********************************************
}} 
CON

        _clkmode  = xtal1 + pll4x
        _xinfreq  = 5_000_000

        irpin     = 8                       'IR receiver module pin used as an input
        
VAR
        byte PropPin_SerOut                 'Variable for P8X32A pin P30
        byte PropPin_SerIn                  'Variable for P8X32A pin P31
        byte PropPin_SerMode                'Variable for P8X32A serial comm format
        word SerPort_BaudRate               'Variable for P8X32A serial comm bit rate
        
OBJ

        term    : "FullDuplexSerial"        'Use Parallax Serial Terminal
        delay   : "timing"
        
PUB start 

  dira[irpin] := %0                         'input on P8, set mode to 0
  
  'Information for serial-communication with PST
  PropPin_SerOut    := 30                   'P30 for USB transmit connection with host PC
  PropPin_SerIn     := 31                   'P31 for USB receive connection with host PC
  PropPin_SerMode   := 0                    'Invert RX mode. See FullDuplexSerial.spin for info.
  SerPort_BaudRate  := 9600                 'Set baud rate at 9600 per second

 'Serial-communication start-up operation
 term.Start(PropPin_SerIn, PropPin_SerOut, PropPin_SerMode, SerPort_BaudRate)

  repeat while ina[irpin] == 1              'wait here until first logic-0 pulse detected

  'IR-input test loop
repeat
    if ina[irpin]                           'test input on irpin
      term.tx(45)                           'no input, print a dash (ASCII 45)
    else                                    'OK, some data present
      term.tx(48)                           'so print a zero (ASCII 48)

 '----- end Program ir_oscope.spin-----