{{
''***********************************************
''*  Program version EX19.2.spin
''*  Author: Jon Titus 12-13-2013 Rev. 2
''*  Program transmits a byte via UART1. 555 timer modulates
''*  byte data and drives an infrared (IR) LED. Modulation at
''*  38 kHz. IR sensor Vishay, type TSOP38238. Program
''*  receives transmitted byte from the sensor and
''*  compares received byte with the byte just
''*  transmitted. Bytes displayed as ASCII characters only
''*  when comparison is true. If false, display "ERROR."
''*  Note "receiver-inversion bit" set for UART1 in
''*  the UART1.start command.
''*  Uses P8X32A Propeller-I board.
''*  Set PST to 9600 bits/sec.
''***********************************************
}}

'Set up clock frequency
CON
    _clkmode = xtal1 + pll16x               'Set MCU clock operation
    _xinfreq = 5_000_000                    'Set for 5 MHz crystal

VAR

    byte char_in                            'Storage location for byte from UART1
    byte index                              'Storage for increasing index value
    
obj

   UART1 : "fullduplexserial"                'Object for IR LED and sensor I/O
   term  : "fullduplexserial"                'Object to control Parallax Serial Terminal
   delays : "timing"                         'Timing object                             
                                                       
pub main
  
  UART1.start(24, 25, %0001, 2400)          'Start UART for IR comms with these settings
                                            'invert received bits, D0 = 1
  term.start(31, 30, %0000, 9600)           'Start PST with these settings 
  
 'Main loop to test IR communications
 repeat

   index := 33                              'Start with 33, 1st printable ASCII character "!"
   repeat while index < 127                 'Repeat this loop until index = 127, ASCII DEL code
     delays.pause1ms(8)                     'Wait 8 milliseconds (arbitrary delay)
    
     UART1.tx(index)                        'Transmit index byte to 555-timer IC                     
     char_in := UART1.rx                    'Get received byte from UART1
                              '
     if (char_in == index)               'Compare sent and received bytes
            term.tx(index)                  'Bytes match (TRUE), so
                                            'Display received byte as ASCII character in PST window

     else                                'If no match (FALSE), print ERROR message
            term.str(String("  ERROR  "))
            UART1.stop                       'Stop UART1, halt operations
            UART1.start(24, 25, %0001, 2400) 'Re-initialize UART1 after detecting an error
      
     index++                                 'Increment index value by 1
    
'---end of Program EX19.2-----