{{
''***********************************************
''*  Program version EX19.3.spin
''*  Author: Jon Titus 01-27-2014. Rev. 2
''*  An MCU or other device transmits a packet via a
''*  Microchip MCP2120 IrDA transceiver chip and a Jaycon
''*  IR module (see text for a schematic diagram).
''*  A separate, equivalent circuit receives the IR transmission
''*  that goes to "UART1" on a Propeller MCU module as defined
''*  below. PST window displays the results.
''*  This program is not in a loop. It runs once. Restart to run again. 
''*  
''*  This program determines what action to take based on the
''*  value returned from the getIRdata program. If no error
''*  occurs, the packet message goes into the array "Data_array".
''*  Data_array[0] indicated the number of bytes in the message.
''*  message goes into array starting at Data_array[1].
""*  
''*  Error codes 1 through 3 cause print out of a message, You can
''*  add code to these sections to take other actions based on
''*  an error. You could send a "resend" command to the
''*  transmitting device to have it try again, and so on.
''*
''*  Uses P8X32A Propeller-I board.
''*  Set PST to 9600 bits/sec.
''***********************************************
}}

'Set up clock frequency
CON _clkmode = xtal1 + pll16x               'Set MCU clock operation
    _xinfreq = 5_000_000                    'Set for 5 MHz crystal


VAR
    byte Bytes_rcvd                         'Used to count bytes for message output.
    byte Array_index                        'Index for Data_array storage
    byte Data_array[256]                    'Set max size of Data_array
    'byte RCVD_check_sum                     'Storage for checksum in a packet
    'byte Check_sum                          'Checksum calculated in getIRdata program
    byte IR_result                          'Returned value from getIRdata program

obj

    term   : "fullduplexserial"             'Object to control Parallax Serial Terminal
    delays : "timing"                       'Timing object                             
    IR     : "getIRdata"                    'Object to handle serial IR input                                                     

pub main
  'Check_sum := 0                            'Initialize calculated checksum to sero
  term.start(31, 30, %0000, 9600)           'Start PST with these settings 

 IR_result := IR.IR_data_input(@Data_array) 'Call IR_data_input object in getIR data   

 case IR_result                             'Test the error value returned from the
                                            'IR_data_input object
    0:                                                  'Error code = 0, all OK
            Bytes_rcvd := Data_array[0]                 'Get number of bytes in msg from Data_array[0]
            repeat Array_index from 1 to Bytes_rcvd     'Loop to print message values in hex
                term.hex(Data_array[Array_index], 2)
                term.tx(13)            
            term.str(String("Bytes received: "))        'Display number of rcvd bytes
            term.dec(Bytes_rcvd)
            term.tx(13)                                 'New line on terminal display
    
    1:                                                  'Error code = 1
        term.str(String("Checksum error!"))
        term.tx(13)

    2:                                                  'Error code = 2
        term.str(String("Incorrect start byte, no bytes received!"))
        term.tx(13)

    3:                                                  'Error code = 3
        term.str(String("Message exceeds array size, no bytes received!"))
        term.tx(13)

    4:
        term.str(String("Byte count equalled zero. Aborted input!"))
        term.tx(13)
        
  OTHER:                                                'IR_data_input returned invalid error code
    term.str(String("Error, not defined."))
 
  '---end of Program EX19.3-----
