{{
''***********************************************
''*  Program: getIRdata.spin
''*  Author: Jon Titus 01-27-2014 Rev. 3
''*  Receive bytes and handle as an IR packet in the format:
''*  start byte ($7E), message-byte count, message bytes,
''*  checksum for only message bytes. The IR_data_input object
''*  receives an array pointer from the calling program or object.
''*  
''*  Return to calling program with an error code:
''*  0 = All OK, data in Data_array, message-byte count in Data_array[0]
''*      message bytes start at Data_array[1].
''*  1 = Checksum error, checksum received does not equal
''*      checksum calculated from rcvd message bytes. No data saved.
''*  2 = First packet byte not $7E, no data saved.
''*  3 = Message byte count exceeds Data_array size limit. No data saved.
''*  4 = Message byte count = 0. No data saved.
''*
''*  Uses "UART1" designation for Propeller UARTn
''***********************************************
}}

VAR

    byte Byte_count                         ' Storage from message byte count value
    byte Array_index                        ' Index variable for Data_array
    byte RCVD_check_sum                     ' Checksum in rcvd packet
    byte Check_sum                          ' Calculated checksum
    byte Trash                              ' Dummy variable for unused data
    byte UART_array_max                     ' Data_array maximum size
                                               
OBJ
  UART1    : "fullduplexserial"             'Object for serial comms with PST
  IRdelays : "timing"                       'Timing object
  
pub IR_data_input (IRdataptr)

    UART1.start(8, 9, %0000, 9600)           'Start IR-input UART with these settings

                                    
    UART1.Rxflush                                       'Clean out old data from the UART
    Check_sum := 0                                      'Initialize check sum to 0
    Byte_count := 0                                     'Initialize byte count to 0
    UART_array_max := 128                               'Set maximum message size
    
        if UART1.rx == $7E                              'Look for $7E as start byte
            Byte_count := UART1.rx                      'OK, so get byte count
            if Byte_count > UART_array_max              'Does byte count exceed array maximum?
                return(3)                               'if so, abort with error code 3
            elseif Byte_count == 0                      'Does Byte_count equal 0? if so abort
                return(4)                               'with error code 4.
                                               
                                                        
           byte[IRdataptr][0] := Byte_count             'Byte_count OK, continue here
            repeat Array_index from 1 to Byte_count     'Set up loop to get message bytes
                byte[IRdataptr][Array_index] := UART1.rx                'Save each byte in the array
                Check_sum := Check_sum + byte[IRdataptr][Array_index]   'Keep a running check sum

            RCVD_check_sum:= UART1.rx                   'Finished the loop, so get received check sum
            if RCVD_check_sum <> Check_sum              'Check sums equal?              
                byte[IRdataptr][0] := 0                 'NO: Set byte count to 0
                return(1)                               '     return with error code 1
            else                                        'YES, so
                byte[IRdataptr][0] := Byte_count        '     put byte count in first array location
                 return (0)                             '     and return with error code (0) no error.

        else                                            'First message byte not $7E
            repeat                                      'Go through this loop to clean more
                Trash := UART1.rx                       'incoming bytes from UART receiver 
                IRdelays.pause1ms(2)                    'wait 2 milliseconds (arbitrary delay)
            until UART1.RxCheck == -1                   'repeat loop until no bytes in UART receiver
            return (2)                                  'return with error code 2
            
 '---end of Program getIRdata-----