''*****************************************************
''*  Program 20.2, Analog-to-Digital Converter and
''*  Distance Measurements, averages 8 samples.
''*  Based on code created by Ken Gracey, Parallax
''*  Modified by John Abshier (jabshier on Forum)
''*  
''*  Modified by Jon Titus, 02-03-2014 Rev. 1
''*  Copyright 2013
''*  Software for a Microchip Technology MCP3202 12-bit       
''*  ADC that displays 7 MSBs on LEDs for voltage on
''*  ADC Channel 0 input, pin 2, Ground input 1,
''*  at pin 3. Not used.                               
''*****************************************************
OBJ
  
  MCP3202   : "MCP3202"             'MCP3202 driver objects here
  delay     : "timing"              'timing.spin file
  
CON

  _CLKMODE      = XTAL1 + PLL4X     'set MCU clock operations
  _XINFREQ      = 5_000_000         '5MHz Crystal

  clock_pin         = 24            'Pin assignments for connections to            '         
  data_pin          = 26            'the MCP3202 ADC from a Propeller
  chip_select_pin   = 25            'P8X32A MCU board
                
VAR    

  long  CH0                         'Variable assignment for Channel-0 data
  long  ADC_sum

PUB Main

 'MCP3202 ADC start-up configuration
 'Enable Channels 0 and 1 for single-ended analog inputs
 'This experiment uses only Channel 0. Ground Channel-1 input.
 'See MCP3202 datasheet for configuration options
 MCP3202.start(data_pin, clock_pin, chip_select_pin, %0011)

 DIRA[23..16] := %11111111                  'Set P8X32A board LED pins as outputs
                                            
repeat                                      'This main loop runs "forever"
  
  ADC_sum := 0                              'Preset the sum of ADC values to zero
  
  repeat 8                                  'Repeat this loop 8 times
      CH0 := MCP3202.in(0)                  'Get digital value for analog signal on Channel 0
      ADC_sum := ADC_sum + CH0              'Add the latest ADC value to ADC_sum
      delay.pause1ms(100)                   '100-msec delay

  ADC_sum := ADC_sum >>3                    'Divide by 8 to get average (same as shift right 3 bits)
  ADC_sum := (ADC_sum >> 5) & %01111111     'Shift 5 bits to right to move 7 MSBs to D6--D0 bit
  OUTA[23..16] := ADC_sum                   'AND with 01111111 to mask out MSB of result to ensure
                                            'MSB in result equals zero. Output to LEDs
  delay.pause1ms(100)                       'Cause a short delay of 100 milliseconds

  '-----end Program 20.2-----      