''*****************************************************
''*  Program 20.3, Lookup Table Approach to distance
''*  measurement swith Sharp sensor GP2Y0A21YK0F.
''*  This routine averages 8 samples and finds distance
''*  in a table of distance values.
''*  Based on code created by Ken Gracey, Parallax
''*  Modified by John Abshier (jabshier on Forum)
''*  
''*  Modified by Jon Titus, 02-05-2014 Rev. 3
''*  Copyright 2013
''*  Software for a Microchip Technology MCP3202 12-bit       
''*  ADC that displays 7 MSBs on LEDs for voltage on
''*  ADC Channel 0 input, pin 2, Ground input 1,
''*  at pin 3. Not used.                               
''*****************************************************
OBJ

  pst       : "FullDuplexSerial"    'serial communication objects here
  MCP3202   : "MCP3202"             'MCP3202 driver objects here
  delay     : "timing"              'timing.spin file
  
CON

  _CLKMODE      = XTAL1 + PLL4X     'set MCU clock operations
  _XINFREQ      = 5_000_000         '5MHz Crystal

  clock_pin         = 24            'Pin assignments for connections to            '         
  data_pin          = 26            'the MCP3202 ADC from a Propeller
  chip_select_pin   = 25            'P8X32A MCU board
                
DAT

'Lookup table data for Dist_cm[x] array
Dist_cm byte 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 82, 75, 68, 63, 58, 54, 51, 48, 45, 43, 41, 39, 37, 35, 34, 32, 31, 30, 29, 28, 27, 26, 25, 25, 24, 23, 22, 22, 21, 21, 20, 20, 19, 19, 18, 18, 17, 17, 17, 17, 16, 16, 16, 15, 15, 15, 14, 14, 14, 14, 13, 13, 13, 13, 13, 12, 12, 12, 12, 12, 11, 11, 11, 11, 11, 11, 11, 10, 10, 10, 10, 10, 10, 10, 10, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 0, 0, 0, 0, 0, 0

VAR    

  long  CH0                         'Variable assignment for Channel-0 data
  long  ADC_sum                     'Variable for sums during averaging math

  byte PropPin_SerOut               'Variable for P8X32A pin P30
  byte PropPin_SerIn                'Variable for P8X32A pin P31
  byte PropPin_SerMode              'Variable for P8X32A serial comm format
  word SerPort_BaudRate             'Variable for P8X32A serial comm bit rate
  
PUB Main

 PropPin_SerOut    := 30            'P30 for USB transmit connection with host PC
 PropPin_SerIn     := 31            'P31 for USB receive connection with host PC
 PropPin_SerMode   := 0             'Invert RX mode. See FullDuplexSerial.spin.
 SerPort_BaudRate  := 9600          'Set baud rate at 9600 per second

 'Serial-communication start-up operation
 pst.Start(PropPin_SerIn, PropPin_SerOut, PropPin_SerMode, SerPort_BaudRate)

 'MCP3202 ADC start-up configuration
 'Enable Channels 0 and 1 for single-ended analog inputs
 'This experiment uses only Channel 0. Ground Channel-1 input.
 'See MCP3202 datasheet for configuration options
 MCP3202.start(data_pin, clock_pin, chip_select_pin, %0011)

 DIRA[23..16] := %11111111                  'Set P8X32A board LED pins as outputs
                                            
repeat                                      'This main loop runs "forever"
  
  ADC_sum := 0                              'Preset the sum of ADC values to zero
  
  repeat 8                                  'Repeat this loop 8 times
      CH0 := MCP3202.in(0)                  'Get digital value for analog signal on Channel 0
      ADC_sum := ADC_sum + CH0              'Add the latest ADC value to ADC_sum
      delay.pause1ms(100)                   '100-msec delay (arbitary, change for faster response

  ADC_sum := ADC_sum >>3                    'Divide by 8 to get average (same as shift right 3 bits)
  ADC_sum := (ADC_sum >> 5) & %01111111     'Shift 5 bits to right to move 7 MSBs to D6--D0 bit
  OUTA[23..16] := ADC_sum                   'AND with 01111111 to mask out MSB of result to ensure
                                            'MSB in result equals zero. Output to LEDs
  pst.tx(1)                                 'Move cursor to home position in PST window   
  pst.str(string("Distance in cm "))        'Display this string in the PST window
  pst.tx(11)                                'Clear data from end of string to end of line
  pst.dec (Dist_cm[ADC_sum])                'Send PST a distance as a decimal number
  pst.Tx(13)                                'Move cursor to a new line
    

  '-----end Program 20.3-----      