''*****************************************************
''*  Program 20.4, Math approach to distance
''*  measurement with Sharp sensor GP2Y0A21YK0F.
''*  This routine averages 8 samples and uses scaled-math
''*  operations to calculate a distance.
''*  Based on code created by Ken Gracey, Parallax
''*  Modified by John Abshier (jabshier on Forum)
''*  
''*  Modified by Jon Titus, 02-05-2014 Rev. 1
''*  Copyright 2013
''*  Software for a Microchip Technology MCP3202 12-bit       
''*  ADC that displays 7 MSBs on LEDs for voltage on
''*  ADC Channel 0 input, pin 2, Ground input 1,
''*  at pin 3. Not used.                               
''*****************************************************
OBJ

  pst       : "FullDuplexSerial"    'serial communication objects here
  MCP3202   : "MCP3202"             'MCP3202 driver objects here
  delay     : "timing"              'timing.spin file
  
CON

  _CLKMODE      = XTAL1 + PLL4X     'set MCU clock operations
  _XINFREQ      = 5_000_000         '5MHz Crystal

  clock_pin         = 24            'Pin assignments for connections to            '         
  data_pin          = 26            'the MCP3202 ADC from a Propeller
  chip_select_pin   = 25            'P8X32A MCU board
                

VAR    

  word  CH0                         'Variable assignment for Channel-0 data
  word  ADC_sum                     'Variable for sums during averaging math
  word  ADC_avg                     'Average of ADC values
  long  Volt_cm                     'Volt-cm constant for calculation
  long  Volts_per_step              'Voltage per setp from ADC for LSB of shifted data
  long  Intercept                   'Y-axis intercept for straight-line equation
  long  Distance                    'cALCULATED DISTANCE IN CENTIMETERS
  
  byte  PropPin_SerOut               'Variable for P8X32A pin P30
  byte  PropPin_SerIn                'Variable for P8X32A pin P31
  byte  PropPin_SerMode              'Variable for P8X32A serial comm format
  word  SerPort_BaudRate             'Variable for P8X32A serial comm bit rate
  
PUB Main

 PropPin_SerOut    := 30            'P30 for USB transmit connection with host PC
 PropPin_SerIn     := 31            'P31 for USB receive connection with host PC
 PropPin_SerMode   := 0             'Invert RX mode. See FullDuplexSerial.spin.
 SerPort_BaudRate  := 9600          'Set baud rate at 9600 per second

 'Serial-communication start-up operation
 pst.Start(PropPin_SerIn, PropPin_SerOut, PropPin_SerMode, SerPort_BaudRate)

 'Scaled constants (multiplied by 2048) for straight-line calculations
 Volt_cm := 42400
 Volts_per_step := 53
 Intercept := 279

 'MCP3202 ADC start-up configuration
 'Enable Channels 0 and 1 for single-ended analog inputs
 'This experiment uses only Channel 0. Ground Channel-1 input.
 'See MCP3202 datasheet for configuration options
 MCP3202.start(data_pin, clock_pin, chip_select_pin, %0011)

 DIRA[23..16] := %11111111                  'Set P8X32A board LED pins as outputs
                                            
repeat                                      'This main loop runs "forever"
  
  ADC_sum := 0                              'Preset the sum of ADC values to zero
  
  repeat 8                                  'Repeat this loop 8 times
      CH0 := MCP3202.in(0)                  'Get digital value for analog signal on Channel 0
      ADC_sum := ADC_sum + CH0              'Add the latest ADC value to ADC_sum
      delay.pause1ms(20)                   '100-msec delay (arbitary, change for faster response

  ADC_avg := ADC_sum >>3                    'Divide SUM by 8 to get average (same as shift right 3 bits)
  ADC_avg := (ADC_avg >> 5) & %01111111     'Shift 5 bits to right to move 7 MSBs to D6--D0 bit
  OUTA[23..16] := ADC_avg                   'AND with 01111111 to mask out MSB of result to ensure
                                            'MSB in result equals zero. Output to LEDs

  if ADC_avg < 15                           'Test ADC value for out-of-range condition
    pst.tx(1)                               'Move cursor to home position in PST window   
    pst.str(string("                    ")) 'Clear the distance reading
    pst.str(string("Out of Range"))         'Print this warning
  else
    Distance := Volt_cm / ((Volts_per_step * ADC_avg) - Intercept) 'Calculate distance

    pst.tx(1)                                 'Move cursor to home position in PST window   
   pst.str(string("Distance in cm "))        'Display this string in the PST window
   pst.tx(11)                                'Clear data from end of string to end of line
   pst.dec (Distance)                        'Send PST a distance as a decimal number
   pst.Tx(13)                                'Move cursor to a new line

  '-----end Program 20.4-----      