{{
''***********************************************
''*  Program 9.1
''*  Author: Jon Titus 12-05-2014 Rev. 1
''*  Copyright 2014
''*  Released under Apache 2 license
''*  Display demonstration that separates a
''*  4-digit value into separate numerals and
''*  uses 74HC595 shift register to control
''*  display segments.
''*  SPI_Asm object library used for SPI-type
''*  output of 8-bit 7-segment patterns
''*  LUMEX LDQ-N516RI 4-digit display module.
''***********************************************
}}

CON _clkmode = xtal1 + pll16x           'Set MCU clock operation
    _xinfreq = 5_000_000                'Set for 5 MHz crystal

OBJ
  SPI     :       "SPI_Asm"             'Invoke the SPI_Asm.spin file
                                        'saved in your Propeller working
                                        'file. Available from the Propeller
                                        'Object exchange (OBEX).
                                            
VAR
    byte segment_code[10]               'Storage for segment codes
    byte count                          'Counter used in display loop
    byte display_loop                   'Counter used for display-loop delay
    byte digit[6]                       'Space for 6 digits
    long big_value                      'Variable for large value to parse 
    byte numb_of_digits                 'Number of digits in big_value
    long divisor                        'Divisor used to start parsing
    byte index                          'Index to identify each digit
    byte decimal_counter                'Counter to track each decimal digit
    byte ser_data                       '8-bit serial data
    byte data_port                      'Data-output pin to shift reg
    byte serial_clock                   'Pulse for shift reg clock
    byte register_clock                 'Pulse for register-transfer clock
                                          
PUB Start                               'Main program starts here

segment_code[0] := %01111110            'Pattern for "0"
segment_code[1] := %00110000            'Pattern for "1"
segment_code[2] := %01101101            'Pattern for "2"
segment_code[3] := %01111001            'Pattern for "3"
segment_code[4] := %00110011            'Pattern for "4"
segment_code[5] := %01011011            'Pattern for "5"
segment_code[6] := %00011111            'Pattern for "6"
segment_code[7] := %01110000            'Pattern for "7"
segment_code[8] := %01111111            'Pattern for "8"
segment_code[9] := %01110011            'Pattern for "9"

''SPI Setup
  SPI.start(15,0)                       'Set SPI for 1 uSec delay (15) and positive-
                                        'going clock pulses (0)

dira[8..10]  := %111                    'Output pins for shift register
outa[8..10]  := %000                    'Set shift-register outputs to 0
dira[11..14] :=%1111                    'Set display-switch outputs
outa[11..14] := %0000                   'Clear display-switch outputs

data_port := 8                          'Use P8 for shift-register data,
serial_clock := 9                       'P9 for shift-register clock, and
register_clock := 10                    'P10 for register flip-flop outputs
                                        
big_value := 3716                       'Large value to work with
numb_of_digits := 4                     'Number of digits in big_value
divisor := 1                            'Starting value of divisor

'LOOP 1: Given the number of digits, calculate the largest divisor.
'  For four digits, start with 1000, for example.

  repeat decimal_counter from 1 to (numb_of_digits - 1)                                
    divisor := divisor * 10
        
 
'LOOP 2: Use divisor to separate the numerals. Use (numb_of_digits - 1)
'  so loop can go from digit[0] to digit[3] for a 4-digit
'  value. Loop stores four values.
 
  repeat index from 0 to (numb_of_digits - 1)
    digit[index] := big_value / divisor
    big_value := big_value - (digit[index] * divisor)
    divisor := divisor / 10

 'LOOP 3: Get the segment data for a numeral and send to shift register.
 'via SPI object. Transfer data to output flip-flops to drive segments,
 'control ULN2003A to turn on corresponding numeral.
 
  repeat                                              'Repeat this loop forever
    repeat display_loop from 0 to 59
        repeat count from 0 to 3                      'Loop for four digits
            ser_data := segment_code[digit[count]]    'Get segment codes
            
            SPI.SHIFTOUT(data_port, serial_clock, SPI#MSBFIRST, 8, ser_data)
            
            outa[register_clock] := 1                 'transfer of bits. Pulse RCLK input
            outa[register_clock] := 0                 'on 74HC595 IC
            outa[11+count] :=1                        'turn on corresponding digit
            waitcnt(clkfreq/200 + cnt)                'time delay
            outa[11+count] := 0                       'turn off the digit
                

 